(use-translation _wizard_ "alterator-install3")

;window definitions
caption "ALT Linux Installer"
full_screen "yes"

;directory with graphic design
(define design-dir "/usr/share/installer/design/current/")

(include-as-is "/dialog/frame")
(include-as-is "/dialog/functions")

;;;;;;;;;;;;;;;;;;;;;;;;;; work with help ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (wizard-with-help?) (not (help-viewer hidden)))

(define (wizard-switch-help)
  (help-viewer hidden (not (help-viewer hidden)))
  (step-list hidden (not (step-list hidden))))

(define (wizard-help-button-text)
  (if (wizard-with-help?) (_wizard_ "Hide help") (_wizard_ "Show help")))

(define (wizard-make-help-button)
  (id 'help-button
      (button (_wizard_ "Show help")
	      (on-click
		(wizard-switch-help)
		(help-button text (wizard-help-button-text))))))

(define (wizard-page-filename url)
  (cond-car (lookout-convert-uri (list url))))


(define (wizard-help-file filename)
  (define (cut-extension name)
    (substring name 0
	       (or (string-index-right name #\.)
		   (string-length name))))
  (or
    (and filename
	 (help-file-content (basename (string-append (cut-extension filename) ".html"))))
    ""))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; listbox selection hacks ;;;;;;;;;;;;;;;;;;;;;
(define *wizard-under-selection* #f)
(define (wizard-protected func)
  (and (not *wizard-under-selection*)
       (begin  (set! *wizard-under-selection* #t)
	       (func)
	       (set! *wizard-under-selection* #f))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; work with steps ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (wizard-get-steps)
  (define (combine-name x num) (string-append x (number->string num)))
  (define (get-description st)
    (let loop ((cmd (mapper-view st)))
      (let ((name (cond-cdr (command-arg-ref cmd 'description))))
	(or name
	    (and (eq? (list-ref cmd 2) 'view)
		 (loop (mapper-view (list-ref cmd 4))))
	    "unknown"))))
  (let loop ((num 0)
	     (result '()))
    (let ((st (global (sure-symbol (combine-name "step" num)))))
      (if st
	(loop (+ num 1) (append result
				(list (cons (get-description st)
					    st))))
	result))))

(define *wizard-steps* (wizard-get-steps)) ;full step-list

;first time initialization
(define *wizard-current-step* (global 'step)); current step name
(define *wizard-current-step-number*
  (list-index (lambda (x) (eq? (cdr x) *wizard-current-step*)) *wizard-steps*))

(define (get-wizard-step num) (cdr (list-ref *wizard-steps* num)))
(define (wizard-get-step-description num) (car (list-ref *wizard-steps* num)))


(define (wizard-next-step)
  (wizard-protected
    (lambda()
      (step-list current (+ (step-list current) 1))
      (dlg-frame-goto (get-wizard-step (step-list current))))))

(define (wizard-prev-step)
  (wizard-protected
    (lambda()
      (step-list current (- (step-list current) 1))
      (dlg-frame-goto (get-wizard-step (step-list current))))))

(define (wizard-last-step? num) (= (- (length *wizard-steps*) 1) num))
(define (wizard-first-step? num) (zero? num))

;;;;;;;;;;;;;;;; step changing ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (wizard-update-pixmaps curr size)
    (if (< curr size)
      (step-list item-pixmap (string-append design-dir "step_state_unknown.png")
		 (cons (+ curr 1) size)))
    (if (< 0 curr)
      (step-list item-pixmap (string-append design-dir "step_state_configured.png")
		 (cons 0 (- curr 1))))
    (step-list item-pixmap (string-append design-dir "step_state_current.png")
	       curr)
    (step-list current curr))

(define (wizard-update-help url)
  (help-viewer text  (wizard-help-file
		       (wizard-page-filename url))))

(define (wizard-update-buttons curr)
  (prev-button text (wizard-prev-button-text curr))
  (next-button text (wizard-next-button-text curr)))

(define (wizard-update-title cur)
  (title-line text (wizard-get-step-description cur)))

(define (wizard-remark-global url)
	  (dlg-frame-global-set!
	    (find (lambda (x) (eq? (cdr x) url))
		  *wizard-steps*)))

;update help content, button texts and title-line
(on-goto
  (wizard-remark-global (sure-symbol url))
  (set! *wizard-current-step-number* (step-list current))
  (let ((curr (step-list current))
	(size (step-list count)))
    (wizard-update-title curr)
    (wizard-update-buttons curr)
    (wizard-update-help url)
    (wizard-update-pixmaps curr size)))

;cleanup our additional events
(on-before-goto
  (page 'delete-event on-next on-prev on-acc-leave on-apply))


;;;;;;;;;;;;;; create widgets ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (wizard-prev-button-text num)
  (if (wizard-first-step? num) (_wizard_ "Cancel") (_wizard_ "Prev")))

;;go previous step button
(define (wizard-make-prev-button)
  (define (on-click-prev)
    (if (dlg-frame-global-passing?)
      (if (wizard-first-step? (step-list current))
	(end-dialog)
	(wizard-prev-step))
      (wizard-protected (lambda () (page 'call on-prev)))))

  (begin-1
    (id 'prev-button
	(button (wizard-prev-button-text *wizard-current-step-number*)
		sizepolicy '(expanding . fixed)
		(dlg-frame-prev-attr-enabled)
		(on-click (on-click-prev))))
    (dlg-frame-mark-prev-button)))


(define (wizard-next-button-text num)
  (if (wizard-last-step? num) (_wizard_ "Finish") (_wizard_ "Next")))

;go next step button
(define (wizard-make-next-button)
  (define (on-click-next)
    (if (dlg-frame-global-passing?)
      (if (wizard-last-step? (step-list current))
	(end-dialog)
	(and (page 'call on-apply) (wizard-next-step)))
      (wizard-protected (lambda () (page 'call on-next)))))

  (begin-1
    (id 'next-button
	(button (wizard-next-button-text *wizard-current-step-number*)
		sizepolicy '(expanding . fixed)
		(dlg-frame-next-attr-enabled)
		(on-click (on-click-next))))
    (dlg-frame-mark-next-button)))

(define (wizard-make-title-line)
  (id 'title-line
      (label (wizard-get-step-description *wizard-current-step-number*)
	     font bold-font)))

;window with internal page and navigation buttons
(define (wizard-make-page-window url)
  (vbox margin 1
	(wizard-make-title-line)
	(dlg-frame-make-page 70 75 url)))

(define (wizard-step-pixmap num)
  (string-append design-dir
		 (cond ((< num *wizard-current-step-number*) "step_state_configured.png")
		       ((= num *wizard-current-step-number*) "step_state_current.png")
		       (else "step_state_unknown.png"))))

(define (wizard-step-labels)
  (fold (lambda (current result)
	  (let ((name (car current))
		(num (length result)))
	    (append result (list (cons name (wizard-step-pixmap num))))))
	'()
	*wizard-steps*))

(define (wizard-make-step-list w h)
  (id 'step-list (listbox 
		   sizepolicy '(fixed . expanding)
		   width w
		   height h
		   (items (wizard-step-labels))
		   current *wizard-current-step-number*
		   (on-select
		     (wizard-protected (lambda()
					 (let ((cur-num (step-list current)))
					   (if (and (dlg-frame-global-passing?)
						    (< cur-num *wizard-current-step-number*))
					     (dlg-frame-goto (get-wizard-step (step-list current)))
					     (step-list current *wizard-current-step-number*)))))))))

;window with help and list with available steps
(define (wizard-make-support-window)
  (vbox margin 1 spacing 1 sizepolicy '(fixed . expanding) width 22
	(wizard-make-step-list 20 90)
	(id 'help-viewer
	    (textbox (wizard-help-file (wizard-page-filename *wizard-current-step*))
		     hidden #t
		     sizepolicy '(fixed . expanding)
		     width 20 readonly yes))))

;widget description
(define (wizard-make-middle url)
  (id 'middle-page
      (hbox
	(relative-width 100)
	(relative-height 75)
	(wizard-make-support-window)
	(wizard-make-page-window url))))

(hbox sizepolicy '(expanding . fixed)
      (hbox sizepolicy '(fixed . fixed)
	    pixmap (string-append design-dir "bg_o_ttr.png"))
      (hbox sizepolicy '(fixed . fixed)
	    pixmap (string-append design-dir "bg_o_tt.png"))
      (hbox sizepolicy '(expanding . fixed)
	    pixmap (string-append design-dir "bg_o_ttr.png")))
(vbox sizepolicy '(expanding . expanding)
      (wizard-make-middle *wizard-current-step*))
(hbox sizepolicy '(expanding . fixed)
      (hbox sizepolicy '(expanding . fixed)
	    margin 2
	    spacing 3
	    pixmap (string-append design-dir "bg_o_bbl.png")
	    (wizard-make-help-button))
      (hbox sizepolicy '(fixed . fixed)
	    pixmap (string-append design-dir "bg_o_bb.png"))
      (hbox sizepolicy '(expanding . fixed)
	    margin 2
	    spacing 3
	    pixmap (string-append design-dir "bg_o_bbr.png")
	    (wizard-make-prev-button)
	    (wizard-make-next-button)))

